import java.util.*;
import java.io.*;

interface Value {
	Value add(Value t);
	Value mult(Value t);
	Value div(Value t);
	Value sub(Value t);
}

class myDouble implements Value{
	double d;
	public myDouble(double d) { this.d = d; }
	public myDouble() { this(0); }
	public myDouble add(Value v){
		return new myDouble(d+((myDouble)v).d);
	}
	public myDouble mult(Value v){
		return new myDouble(d*((myDouble)v).d);
	}
	public myDouble sub(Value v){
		return new myDouble(d-((myDouble)v).d);
	}
	public myDouble div(Value v){
		return new myDouble(d/((myDouble)v).d);
	}
	public String toString() { return ""+d; }
}

class myInt implements Value {
	int d;
	public myInt(int d) { this.d = d; }
	public myInt() { this(0); }
	public myInt add(Value v){
		return new myInt(d+((myInt)v).d);
	}
	public myInt mult(Value v){
		return new myInt(d*((myInt)v).d);
	}
	public myInt sub(Value v){
		return new myInt(d-((myInt)v).d);
	}
	public myInt div(Value v){
		return new myInt(d/((myInt)v).d);
	}
	public String toString() { return ""+d; }
}

class Rational implements Value {
        private int x,y;
        public Rational(int x, int y){ this.x=x; this.y=y; }
        public Rational(int p){ this(p,1); }
        private static int pgcd(int a, int b){
                a = a<0 ? -a: a;
                b = b<0 ? -b: b;
                if(a==0 || b==0) return a+b;
                while(a!=b){
                        if(a>b) a = a-b;
                        if(b>a) b = b-a;
                }
                return a;
        }
        public Rational simplify(){
                return new Rational(x/pgcd(x,y),y/pgcd(x,y));
        }

        public String toString(){
                return "("+x+"/"+y+")";
        }

        public Rational add(Rational r){
                return new Rational(x*r.y+y*r.x,y*r.y).simplify();
        }

        public Rational mult(Rational r){
                return new Rational(x*r.x,y*r.y).simplify();
        }

        public Rational inv(){
                return new Rational(y,x);
        }
	public Rational add(Value v){ return add((Rational)v); }
	public Rational sub(Value v){ return sub((Rational)v); }
	public Rational div(Value v){ return div((Rational)v); }
	public Rational mult(Value v){ return mult((Rational)v); }

        public Rational div(Rational r) { return mult(r.inv()); }

	public Rational sub(Rational r) { return add(r.mult(new Rational(-1))); }

}


class Calc {
	Deque<Value> data = new LinkedList<>();
	void eval(String op){
		Value b = data.removeLast();
		Value a = data.removeLast();
		switch(op.charAt(0)){
			case '+':
				data.addLast(a.add(b));
				break;
			case '-':
				data.addLast(a.sub(b));
				break;
			case '*':
				data.addLast(a.mult(b));
				break;
			case '/':
				data.addLast(a.div(b));
				break;
			default:
				throw new UnsupportedOperationException();

		}
	}
	public static void main(String [] args){
		Calc mycalc = new Calc();
		Scanner s = new Scanner(System.in);
		System.out.println("What type of calculator is this? 1=Int, 2=Double, 3=Rational");
		int choice = s.nextInt();
		System.out.println("Enter number, operation, or Ctl-D to exit");
		while(s.hasNext()){
			if(s.hasNextInt()){
				Value v=null;
				if(choice == 1) v = new myInt(s.nextInt());
				if(choice == 2) v = new myDouble(s.nextInt());
				if(choice == 3) v = new Rational(s.nextInt());
				mycalc.data.addLast(v);
			}else{
				try{
					mycalc.eval(s.next());
				}catch(NoSuchElementException e){
					System.out.println("ERROR! : Not enough elements in stack for operation!");
					return;
				}catch(UnsupportedOperationException e){
					System.out.println("ERROR! : Unknown operation!");
					return;
				}
			}
			System.out.println("Current state:");
			System.out.println(mycalc.data);
		}
	}
}
